@@ -1,6 +1,21 @@
 #[macro_use]
 mod macros;
 
+mod utils {
+    use std::{fmt::Display, io::Write};
+
+    pub fn writeln<W: Write, D: Display>(writer: &mut W, item: D) -> Result<(), std::io::Error> {
+        write!(writer, "{}\n", item)
+    }
+
+    pub fn select_slice<'a>(
+        raw: &'a Option<Vec<u8>>,
+        offset_length: &'a (isize, usize),
+    ) -> Option<&'a [u8]> {
+        raw.as_ref()
+            .map(|vec| &vec[offset_length.0 as usize..(offset_length.0 as usize + offset_length.1)])
+    }
+}
 #[allow(non_snake_case)]
 #[allow(unused_parens)]
 #[allow(clippy::new_without_default)]
@@ -171,6 +186,7 @@ mod test {
     use super::server::*;
     use crate::model::headers::*;
     use std::convert::TryFrom;
+    use std::convert::TryInto;
 
     #[test]
     fn parses_stomp_frame() {
@@ -234,6 +250,61 @@ mod test {
         );
     }
 
+    #[test]
+    fn writes_message_frame_bytes() {
+        let body = b"Lorem ipsum dolor sit amet,".to_vec();
+
+        let frame = MessageFrame::new(
+            MessageIdValue::new("msg-1".to_owned()),
+            DestinationValue::new("path/to/hell".to_owned()),
+            SubscriptionValue::new("annual".to_owned()),
+            Some(ContentTypeValue::new("foo/bar".to_owned())),
+            None,
+            body,
+        );
+
+        let bytes: Vec<u8> = frame.try_into().expect("Error writing bytes");
+
+        assert_eq!(
+            b"MESSAGE\n\
+            message-id:msg-1\n\
+            destination:path/to/hell\n\
+            subscription:annual\n\
+            content-type:foo/bar\n\
+            \n\
+            Lorem ipsum dolor sit amet,\x00",
+            bytes.as_slice()
+        );
+    }
+
+    #[test]
+    fn writes_binary_message_frame() {
+        let body = vec![0, 1, 1, 2, 3, 5, 8, 13];
+
+        let frame = MessageFrame::new(
+            MessageIdValue::new("msg-1".to_owned()),
+            DestinationValue::new("path/to/hell".to_owned()),
+            SubscriptionValue::new("annual".to_owned()),
+            Some(ContentTypeValue::new("foo/bar".to_owned())),
+            None,
+            body,
+        );
+
+        let bytes: Vec<u8> = frame.try_into().expect("Error writing bytes");
+
+        assert_eq!(
+            b"MESSAGE\n\
+            message-id:msg-1\n\
+            destination:path/to/hell\n\
+            subscription:annual\n\
+            content-type:foo/bar\n\
+            \n\
+            \x00\x01\x01\x02\x03\x05\x08\x0d\
+            \x00",
+            bytes.as_slice()
+        );
+    }
+
     #[test]
     fn parses_send_frame() {
         let message = b"SEND\n\
@@ -251,4 +322,20 @@ mod test {
             panic!("Send Frame not parsed correctly");
         }
     }
+
+    #[test]
+    fn parses_binary_send_frame() {
+        let message = b"SEND\n\
+            destination:stairway/to/heaven\n\
+            \n\
+            \x00\x01\x01\x02\x03\x05\x08\x0d\
+            \x00"
+            .to_vec();
+
+        if let Ok(ClientFrame::Send(frame)) = ClientFrame::try_from(message) {
+            assert_eq!(&[0u8, 1, 1, 2, 3, 5, 8, 13], frame.body().unwrap());
+        } else {
+            panic!("Send Frame not parsed correctly");
+        }
+    }
 }
